﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using System.Diagnostics;
using System.Xml;
using System.IO;
using System.Collections;

namespace LINQDemos.LinqToXML
{
    public static class LinqtoXmlSample
    {

        public static void TestXmlGeneration()
        {
            GenerateXmlOldSchoolWithXmlTextWriter();
            GenerateXmlOldSchoolWithXmlDocument();
            GenerateXmlFromLinq();
        }

        public static void GenerateXmlFromLinq()
        {
            Console.WriteLine("Linq");

            XElement processes = new XElement("processes",
                 from p in Process.GetProcesses()
                 orderby p.ProcessName
                 where p.PeakPagedMemorySize64 > 10000000
                 select new XElement("process",
                            new XAttribute("name", p.ProcessName),
                            new XAttribute("peakPageMemory", p.PeakPagedMemorySize64)));

            Console.WriteLine(processes.ToString());

            Console.WriteLine();
        }

        public static void GenerateXmlOldSchoolWithXmlTextWriter()
        {
            Console.WriteLine("Old School - XmlTextWriter");
            Process[] runningProcesses = Process.GetProcesses();
            ArrayList memoryPigs = new ArrayList(runningProcesses.Length);

            foreach (Process p in runningProcesses)
            {
                if (p.PeakPagedMemorySize64 > 10000000)
                {
                    memoryPigs.Add(p);
                }
            }

            memoryPigs.Sort(new ProcessComparer());

            StringBuilder sb = new StringBuilder();
            using (StringWriter sw = new StringWriter(sb))
            {
                using (XmlTextWriter writer = new XmlTextWriter(sw))
                {
                    writer.Formatting = Formatting.Indented;
                    writer.WriteStartElement("processes");
                    foreach (Process p in memoryPigs)
                    {
                        writer.WriteStartElement("process");
                        writer.WriteAttributeString("name", p.ProcessName);
                        writer.WriteAttributeString("peakPageMemory", p.PeakPagedMemorySize64.ToString());
                        writer.WriteEndElement();
                    }
                    writer.WriteEndElement();
                    writer.Flush();
                }
            }
            Console.WriteLine(sb.ToString());

            Console.WriteLine();
        }

        public static void GenerateXmlOldSchoolWithXmlDocument()
        {
            Console.WriteLine("Old School - XmlDocument");

            Process[] runningProcesses = Process.GetProcesses();
            ArrayList memoryPigs = new ArrayList(runningProcesses.Length);

            foreach (Process p in runningProcesses)
            {
                if (p.PeakPagedMemorySize64 > 10000000)
                {
                    memoryPigs.Add(p);
                }
            }

            memoryPigs.Sort(new ProcessComparer());

            XmlDocument doc = new XmlDocument();
            XmlElement processes = doc.CreateElement("processes");
            foreach (Process p in memoryPigs)
            {
                XmlElement process = doc.CreateElement("process");
                XmlAttribute name = doc.CreateAttribute("name");
                name.Value = p.ProcessName;
                process.Attributes.Append(name);
                XmlAttribute memory = doc.CreateAttribute("peakPageMemory");
                memory.Value = p.PeakPagedMemorySize64.ToString();
                process.Attributes.Append(memory);
                processes.AppendChild(process);
            }
            doc.AppendChild(processes);

            Console.WriteLine(doc.OuterXml);

            Console.WriteLine();
        }



    }


    public class ProcessComparer : IComparer
    {

        #region IComparer Members

        public int Compare(object x, object y)
        {
            if (x is Process && y is Process)
            {
                return String.Compare(((Process)x).ProcessName, ((Process)y).ProcessName);
            }
            else
            {
                throw new ArgumentException("This comparer only compares Process objects.");
            }
        }

        #endregion
    }
}
